/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.TestCaseConfig;
import com.eviware.soapui.config.TestSuiteConfig;
import com.eviware.soapui.impl.wsdl.actions.testsuite.AddNewTestCaseAction;
import com.eviware.soapui.impl.wsdl.actions.testsuite.RemoveTestSuiteAction;
import com.eviware.soapui.impl.wsdl.actions.testsuite.RenameTestSuiteAction;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.testsuite.TestSuiteListener;
import com.eviware.soapui.model.testsuite.TestCase.TestStatus;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.tree.nodes.TestSuiteTreeNode;

/**
 * TestSuite implementation for WSDL projects.
 * 
 * @author Ole.Matzura
 */

public class WsdlTestSuite extends AbstractWsdlModelItem implements TestSuite
{
   private final WsdlProject project;
   private final TestSuiteConfig config;
   private List<WsdlTestCase> testCases = new ArrayList<WsdlTestCase>();
   private ImageIcon testSuiteIcon;
   private Set<TestSuiteListener> listeners = new HashSet<TestSuiteListener>();

   public WsdlTestSuite(WsdlProject project, TestSuiteConfig config)
   {
      this.project = project;
      this.config = config;
      
      TestCaseConfig [] testCaseConfigs = config.getTestCaseArray();
      for (int i = 0; i < testCaseConfigs.length; i++)
      {
         testCases.add( new WsdlTestCase( this, testCaseConfigs[i] ));
      }
      
      testSuiteIcon = SoapUI.createImageIcon("/testSuite.gif");
      
      addAction( new AddNewTestCaseAction( this ) );
      addAction( new RenameTestSuiteAction( this ) );
      addAction( new RemoveTestSuiteAction( this ) );
   }

   public Project getProject()
   {
      return project;
   }

   public int getTestCaseCount()
   {
      return testCases.size();
   }

   public TestCase getTestCaseAt(int index)
   {
      return testCases.get( index );
   }

   public String getName()
   {
      return config.getName();
   }

   public ImageIcon getIcon()
   {
      return testSuiteIcon;
   }

   public void setName(String name)
   {
      String old = getName();
      config.setName( name );
      notifyPropertyChanged(NAME_PROPERTY, old, name);
   }
   
   public WsdlTestCase addNewTestCase( String name )
   {
      WsdlTestCase testCase = new WsdlTestCase( this, config.addNewTestCase());
      testCase.setName( name );
      testCases.add( testCase );
      notifyTestCaseAdded( testCase );

      return testCase;
   }
   
   public void removeTestCase(WsdlTestCase testCase )
   {
      int ix = testCases.indexOf( testCase );

      notifyTestCaseRemoved( testCase );

      testCases.remove( ix );
      config.removeTestCase( ix );
   }

   protected SoapUITreeNode createTreeNode()
   {
      return new TestSuiteTreeNode( this );
   }
   
   public void notifyTestCaseAdded( WsdlTestCase testCase )
   {
      TestSuiteListener[] a = listeners.toArray( new TestSuiteListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testCaseAdded( testCase );
      }
   }
   
   public void notifyTestCaseRemoved( WsdlTestCase testCase )
   {
   	TestSuiteListener[] a = listeners.toArray( new TestSuiteListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testCaseRemoved( testCase );
      }
   } 

   public void notifyTestStepAdded( WsdlTestStep testStep, int index )
   {
   	TestSuiteListener[] a = listeners.toArray( new TestSuiteListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testStepAdded( testStep, index );
      }
   }
   
   public void notifyTestStepRemoved( WsdlTestStep testStep, int ix )
   {
   	TestSuiteListener[] a = listeners.toArray( new TestSuiteListener[listeners.size()] );
      
      for (int c = 0; c < a.length; c++ )
      {
         a[c].testStepRemoved( testStep, ix );
      }
   } 
   
	public void addTestSuiteListener(TestSuiteListener listener)
	{
		listeners.add( listener );
	}

	public void removeTestSuiteListener(TestSuiteListener listener)
	{
		listeners.remove( listener );
	} 
   

}
