/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support;

import java.io.StringWriter;
import java.util.Map;

import javax.wsdl.BindingOperation;
import javax.wsdl.Part;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.soap.SOAPBody;
import javax.wsdl.extensions.soap.SOAPHeader;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xmlbeans.SchemaGlobalElement;
import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.RequestBuilder;
import com.eviware.soapui.support.XmlUtils;

/**
 * Builds SOAP requests according to WSDL/XSD definitions
 * 
 * @author Ole.Matzura
 */

public class SoapRequestBuilder implements RequestBuilder
{
   private final static Log log = LogFactory.getLog( SoapRequestBuilder.class );

   private final QName envelopeQName  = new QName("http://schemas.xmlsoap.org/soap/envelope/", "Envelope");

   private final QName bodyQName = new QName("http://schemas.xmlsoap.org/soap/envelope/", "Body");

   private final QName headerQName = new QName("http://schemas.xmlsoap.org/soap/envelope/", "Header");

   private WsdlContext wsdlContext;

   private WsdlInterface iface;
   
   public SoapRequestBuilder(WsdlInterface iface) throws Exception
   {
      this.iface = iface;
      this.wsdlContext = iface.getWsdlContext();
   }

   public SoapRequestBuilder(WsdlContext wsdlContext)
   {
      this.wsdlContext = wsdlContext;
   }
   
   public Interface getInterface()
   {
      return iface;
   }

   public Request buildRequest(Operation operation, Map params)
   {
      return null;
   }

   public String buildSoapRequest(BindingOperation bindingOperation, boolean buildOptional ) throws Exception
   {
      boolean inputSoapEncoded = WsdlUtils.isInputSoapEncoded( bindingOperation );
      SampleXmlUtil xmlGenerator = new SampleXmlUtil(inputSoapEncoded);
      xmlGenerator.setIgnoreOptional( !buildOptional );
      
      XmlObject object = XmlObject.Factory.newInstance();
      XmlCursor cursor = object.newCursor();
      cursor.toNextToken();
      cursor.beginElement(envelopeQName);

      if( inputSoapEncoded )
      {
         cursor.insertNamespace( "xsi", "http://www.w3.org/2001/XMLSchema-instance" );
         cursor.insertNamespace( "xsd", "http://www.w3.org/2001/XMLSchema" );
      }
      
      cursor.toFirstChild();

      cursor.beginElement(bodyQName);
      cursor.toFirstChild();

      if( WsdlUtils.isRpc( wsdlContext.getDefinition(), bindingOperation ))
      {
         buildRpcRequest( bindingOperation, cursor, xmlGenerator );
      }
      else 
      {
         buildDocumentRequest( bindingOperation, cursor, xmlGenerator );
      }   
      
      addHeaders(bindingOperation, cursor, xmlGenerator);
      cursor.dispose();

      try
      {
         StringWriter writer = new StringWriter();
         XmlUtils.serializePretty( object, writer );
         return writer.toString();
      }
      catch( Exception e )
      {
         e.printStackTrace();
         return object.xmlText();
      }
   }

   private void addHeaders(BindingOperation bindingOperation, XmlCursor cursor, SampleXmlUtil xmlGenerator) throws Exception
   {
      ExtensibilityElement[] headers = WsdlUtils.getExtensiblityElements(bindingOperation.getBindingInput()
            .getExtensibilityElements(), SOAPHeader.class);
      if (headers.length > 0)
      {
         // reposition
         cursor.toStartDoc();
         cursor.toChild(envelopeQName);
         cursor.toFirstChild();
         
         cursor.beginElement(headerQName);
         cursor.toFirstChild();

         for (int i = 0; i < headers.length; i++)
         {
            SOAPHeader header = (SOAPHeader) headers[i];
            
      //      String partName = header.getPart();

            Part part = wsdlContext.getDefinition().getMessage( header.getMessage() ).getPart( header.getPart() );

            //Part part = bindingOperation.getOperation().getInput().getMessage().getPart(partName);

            if( part != null )
            	createElementForPart( part, cursor, xmlGenerator );
            else
            	log.error( "Missing part for header; " + header.getPart() );
         }
      }
   }

   private void createElementForPart(Part part, XmlCursor cursor, SampleXmlUtil xmlGenerator ) throws Exception
   {
      QName elementName = part.getElementName();
      QName typeName = part.getTypeName();
      
      if( elementName != null )
      {
         cursor.beginElement(elementName);
   
         if( wsdlContext.hasSchemaTypes() )
         {
            SchemaGlobalElement elm = wsdlContext.getSchemaTypes().findElement(elementName);
            if( elm != null )
            {
               cursor.toFirstChild();
               xmlGenerator.createSampleForType(elm.getType(), cursor);
               cursor.toParent();
            }
         }
      }
      else 
      {
         cursor.beginElement( new QName( wsdlContext.getDefinition().getTargetNamespace(), part.getName() ));
         if( typeName != null && wsdlContext.hasSchemaTypes() )
         {
            SchemaType type = wsdlContext.getSchemaTypes().findType( typeName );
            
            if( type != null )
            {
               cursor.toFirstChild();
               xmlGenerator.createSampleForType( type, cursor );
            }
            else log.error( "Could not find type [" + typeName + "] specified in part [" + part.getName() + "]" );
         }
      }
   }

   private void buildDocumentRequest(BindingOperation bindingOperation, XmlCursor cursor, SampleXmlUtil xmlGenerator) throws Exception
   {
      Part[] parts = WsdlUtils.getInputParts( bindingOperation );
      if( parts.length > 1 )
         log.warn( "document style operation should not have more than one part" );
      
      for (int i = 0; i < parts.length; i++)
      {
         XmlCursor c = cursor.newCursor();
         c.toLastChild();
         createElementForPart( parts[i], c, xmlGenerator );
         c.dispose();
      }
   }

   private void buildRpcRequest(BindingOperation bindingOperation, XmlCursor cursor, SampleXmlUtil xmlGenerator) throws Exception
   {
      // rpc requests use the operation name as root element
      SOAPBody body = (SOAPBody) WsdlUtils.getExtensiblityElement(bindingOperation.getBindingInput()
            .getExtensibilityElements(), SOAPBody.class);
      
      String ns = wsdlContext.getDefinition().getTargetNamespace();
      if( body != null && body.getNamespaceURI() != null )
         ns = body.getNamespaceURI();
      
      cursor.beginElement( new QName( ns, bindingOperation.getName() ));
      if( xmlGenerator.isSoapEnc() )
         cursor.insertAttributeWithValue( new QName( "http://schemas.xmlsoap.org/soap/envelope/", "encodingStyle" ), 
            "http://schemas.xmlsoap.org/soap/encoding/");
      
      Part[] inputParts = WsdlUtils.getInputParts( bindingOperation );
      for (int i = 0; i < inputParts.length; i++)
      {
         Part part = inputParts[i];
         XmlCursor c = cursor.newCursor();
         c.toLastChild();
         c.insertElement( part.getName() );
         c.toPrevToken();
         
         if( wsdlContext.hasSchemaTypes() )
         {
            SchemaType type = wsdlContext.findType( part.getTypeName() );
            if( type != null )
               xmlGenerator.createSampleForType( type, c );
            else
               log.error( "Failed to find type [" + part.getTypeName() + "]" );
         }
         
         c.dispose();
      }
   }
   
   public void setWsdlContext( WsdlContext wsdlContext )
   {
   	this.wsdlContext = wsdlContext;
   }
}
