/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Message;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.Service;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.soap.SOAPAddress;
import javax.wsdl.extensions.soap.SOAPBinding;
import javax.wsdl.extensions.soap.SOAPBody;
import javax.wsdl.extensions.soap.SOAPFault;
import javax.wsdl.extensions.soap.SOAPOperation;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;

/**
 * Wsdl-related tools
 * 
 * @author Ole.Matzura
 */

public class WsdlUtils
{
   private final static Logger log = Logger.getLogger( WsdlUtils.class );
   
   public static ExtensibilityElement getExtensiblityElement(List list, Class clazz )
   {
      ExtensibilityElement[] elements = getExtensiblityElements( list, clazz );
      return elements.length > 0 ? elements[0] : null;
   }
   
   public static ExtensibilityElement [] getExtensiblityElements(List list, Class clazz )
   {
      List<ExtensibilityElement> result = new ArrayList<ExtensibilityElement>();
      
      for( Iterator<ExtensibilityElement> i = list.iterator(); i.hasNext(); )
      {
         ExtensibilityElement elm = (ExtensibilityElement) i.next();
         if( clazz.isAssignableFrom( elm.getClass() ) )
         {
            result.add( elm );
         }
      }

      return result.toArray( new ExtensibilityElement[result.size()] );
   }
   
   public static String getSoapAction(BindingOperation operation)
   {
      List list = operation.getExtensibilityElements();
      SOAPOperation soapOperation = (SOAPOperation) WsdlUtils.getExtensiblityElement( list, SOAPOperation.class );

      return soapOperation == null ? null : soapOperation.getSoapActionURI();
   }
   
   public static String[] getEndpointsForBinding(Definition definition, Binding binding)
   {
      List<String> result = new ArrayList<String>(); 
      Map map = definition.getServices();
      for( Iterator i = map.values().iterator(); i.hasNext(); )
      {
         Service service = (Service) i.next();
         Map portMap = service.getPorts();
         for( Iterator i2 = portMap.values().iterator(); i2.hasNext(); )
         {
            Port port = (Port) i2.next();
            if( port.getBinding() == binding )
            {
               SOAPAddress soapAddress = (SOAPAddress) WsdlUtils.getExtensiblityElement( port.getExtensibilityElements(), SOAPAddress.class );
               result.add( soapAddress.getLocationURI() );
            }
         }
      }
      
      return result.toArray( new String[result.size()]);
   }
   
   public static Binding findBindingForOperation(Definition definition, BindingOperation bindingOperation)
   {
      Map services = definition.getServices();
      Iterator<Service> s = services.values().iterator();
      
      while( s.hasNext())
      {
         Map ports = s.next().getPorts();
         Iterator<Port> p = ports.values().iterator();
         while( p.hasNext())
         {
            Binding binding = p.next().getBinding();
            List bindingOperations = binding.getBindingOperations();
            for (Iterator iter = bindingOperations.iterator(); iter.hasNext();)
            {
               BindingOperation op = (BindingOperation) iter.next();
               if( op.getName().equals( bindingOperation.getName() ))
                  return binding;
            }
         }
      }
      
      Map bindings = definition.getBindings();
      Iterator<QName> names = bindings.keySet().iterator();
      while( names.hasNext() ) 
      {
      	Binding binding = definition.getBinding( names.next() );
      	List bindingOperations = binding.getBindingOperations();
         for (Iterator iter = bindingOperations.iterator(); iter.hasNext();)
         {
            BindingOperation op = (BindingOperation) iter.next();
            if( op.getName().equals( bindingOperation.getName() ))
               return binding;
         }
      }
      
      return null;
   }
   
   public static boolean isInputSoapEncoded(BindingOperation bindingOperation)
   {
      SOAPBody body = (SOAPBody) WsdlUtils.getExtensiblityElement(bindingOperation.getBindingInput()
            .getExtensibilityElements(), SOAPBody.class);
      
      return body != null && body.getUse() != null && body.getUse().equalsIgnoreCase( "encoded" ) &&
           body.getEncodingStyles().contains( "http://schemas.xmlsoap.org/soap/encoding/" );
   }
   
   public static boolean isOutputSoapEncoded(BindingOperation bindingOperation)
   {
      SOAPBody body = (SOAPBody) WsdlUtils.getExtensiblityElement(bindingOperation.getBindingOutput()
            .getExtensibilityElements(), SOAPBody.class);
      
      return body != null && body.getUse() != null && body.getUse().equalsIgnoreCase( "encoded" ) &&
           body.getEncodingStyles().contains( "http://schemas.xmlsoap.org/soap/encoding/" );
   }
   
   public static boolean isRpc(Definition definition, BindingOperation bindingOperation)
   {
      SOAPOperation soapOperation = (SOAPOperation) WsdlUtils.getExtensiblityElement( 
            bindingOperation.getExtensibilityElements(), SOAPOperation.class );
      
      if( soapOperation != null && soapOperation.getStyle() != null )
         return soapOperation.getStyle().equalsIgnoreCase("rpc");
      
      Binding binding = findBindingForOperation( definition, bindingOperation );
      if( binding == null ) 
      {
         log.error( "Failed to find binding for operation [" + bindingOperation.getName() + "] in definition [" + 
               definition.getDocumentBaseURI() + "]" );
         return false;
      }
      
      SOAPBinding soapBinding = (SOAPBinding) WsdlUtils.getExtensiblityElement( 
            binding.getExtensibilityElements(), SOAPBinding.class );
      
      return soapBinding != null && soapBinding.getStyle() != null && soapBinding.getStyle().equalsIgnoreCase( "rpc" );
   }
   
   public static Part[] getInputParts(BindingOperation operation)
   {
      List<Part> result = new ArrayList<Part>();
      Message msg = operation.getOperation().getInput().getMessage();
      SOAPBody body = (SOAPBody) WsdlUtils.getExtensiblityElement(operation.getBindingInput()
            .getExtensibilityElements(), SOAPBody.class);

      if (body == null || body.getParts() == null)
      {
         result.addAll( msg.getOrderedParts( null ));
      }
      else
      {
         Iterator i = body.getParts().iterator();
         while (i.hasNext())
         {
            String partName = (String) i.next();
            Part part = msg.getPart( partName );
            
            result.add(part);
         }
      }

      return result.toArray(new Part[result.size()]);
   }
   
   public static Part[] getFaultParts(BindingOperation bindingOperation, String faultName)
   {
      List<Part> result = new ArrayList<Part>();
      
      SOAPFault fault = (SOAPFault) WsdlUtils.getExtensiblityElement(bindingOperation.getBindingFault( faultName )
            .getExtensibilityElements(), SOAPFault.class);

      if (fault != null )
      {
         result.addAll( bindingOperation.getOperation().getFault( fault.getName() ).getMessage().getOrderedParts( null ));
      }
      
      return result.toArray(new Part[result.size()]);
   }
   
   public static Part[] getOutputParts(BindingOperation operation)
   {
      List<Part> result = new ArrayList<Part>();
      Message msg = operation.getOperation().getOutput().getMessage();
      SOAPBody body = (SOAPBody) WsdlUtils.getExtensiblityElement(operation.getBindingOutput()
            .getExtensibilityElements(), SOAPBody.class);

      if (body == null || body.getParts() == null)
      {
         result.addAll( msg.getOrderedParts( null ));
      }
      else
      {
         Iterator i = body.getParts().iterator();
         while (i.hasNext())
         {
            String partName = (String) i.next();
            Part part = msg.getPart( partName );
            
            result.add(part);
         }
      }

      return result.toArray(new Part[result.size()]);
   }
}
