/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import org.apache.commons.beanutils.PropertyUtils;

/**
 * Table for displaying property name/value pairs
 * 
 * @author Ole.Matzura
 */

public class PropertiesTable extends JPanel
{
	private PropertiesTableModel tableModel;

	public PropertiesTable( String title, Object propertyObject )
	{
		super( new BorderLayout());
		
		tableModel = new PropertiesTableModel( propertyObject );
		JTable table = new JTable( tableModel );
      table.getColumnModel().getColumn(0).setHeaderValue( "Property" );
      table.getColumnModel().getColumn(1).setHeaderValue( "Value" );
      table.getColumnModel().getColumn(1).setCellRenderer( new PropertiesTableCellRenderer() );
      
      add( new JScrollPane( table ), BorderLayout.CENTER );
      TitledBorder titledBorder = BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), title );
      titledBorder.setTitleFont( titledBorder.getTitleFont().deriveFont( Font.PLAIN, 11 ));
		setBorder( titledBorder);
		
		table.setBackground( Color.WHITE );
	}
	
	public PropertiesTableModel getTableModel()
	{
		return tableModel;
	}

	public void addProperty( String caption, String name)
	{
		addProperty( caption, name, false  );
	}
	
	public void addProperty( String caption, String name, boolean editable )
	{
		tableModel.addProperty( caption, name, editable  );
	}
	
	private class PropertiesTableModel extends AbstractTableModel implements PropertyChangeListener 
   {
		private List<PropertyDescriptor> properties = new ArrayList<PropertyDescriptor>();
		private final Object propertyObject;
		
		public PropertiesTableModel(Object propertyObject)
		{
			this.propertyObject = propertyObject;
		}
		
		public void addProperty( String caption, String name, boolean editable )
		{
			properties.add( new PropertyDescriptor( caption, name, editable ));
		}
		
		public int getRowCount()
		{
			return properties.size();
		}

		public int getColumnCount()
		{
			return 2;
		}
		
		public boolean isCellEditable(int rowIndex, int columnIndex)
		{
			if( columnIndex == 0 ) return false;
			return properties.get( rowIndex ).isEditable() && 
				PropertyUtils.isWriteable( propertyObject, properties.get( rowIndex ).getName() );
		}
		
		public void setValueAt(Object aValue, int rowIndex, int columnIndex)
		{
			try
			{
				if( columnIndex == 1 && properties.get( rowIndex ).isEditable() )
					PropertyUtils.setSimpleProperty( propertyObject, properties.get( rowIndex ).getName(), aValue );
			}
			catch ( Exception e)
			{
				e.printStackTrace();
			}
		}

		public Object getValueAt(int rowIndex, int columnIndex)
		{
			try
			{
				switch (columnIndex)
				{
				case 0:
					return properties.get( rowIndex ).getCaption();
				case 1:
					return PropertyUtils.getSimpleProperty(propertyObject,
							properties.get(rowIndex).getName());
				}
			}
			catch (Exception e)
			{
				e.printStackTrace();
				return e.getMessage();
			}
			
			return null;
		}

		public void propertyChange(PropertyChangeEvent evt)
		{
			String name = evt.getPropertyName();
			if( name.lastIndexOf( '@') > 0 ) 
				name = name.substring( name.lastIndexOf( '@') + 1);
			
			for( int c = 0; c < properties.size(); c++ )
			{
				if( properties.get( c ).getName().equals( name ))
				{
					fireTableCellUpdated( c, 1 );
					return;
				}
			}
		}}
	
	private class PropertyDescriptor
	{
		private final String caption;
		private final String name;
		private final boolean editable;

		public PropertyDescriptor(String caption, String name, boolean editable)
		{
			this.caption = caption;
			this.name = name;
			this.editable = editable;
		}

		public String getCaption()
		{
			return caption;
		}

		public boolean isEditable()
		{
			return editable;
		}

		public String getName()
		{
			return name;
		}
	}

	private class PropertiesTableCellRenderer extends DefaultTableCellRenderer
	{
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
		{
			Component component = super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
					row, column);
			
			if( component instanceof JComponent && value != null )
				((JComponent)component).setToolTipText( value.toString() );
			
			return component;
		}
		
	}
}
